<?php
if (!defined('DC_CONTEXT_ADMIN')) { return; }

dcCore::app()->menu[dcAdmin::MENU_PLUGINS]->addItem(
    __('Image Meta Extender'),
    'plugin.php?p=dcImageMetaExtender',
    'index.php?pf=dcImageMetaExtender/icon.png',
    preg_match('/plugin.php\?p=dcImageMetaExtender(&.*)?$/', $_SERVER['REQUEST_URI']),
    dcCore::app()->auth->check(
        dcCore::app()->auth->makePermissions([
            dcAuth::PERMISSION_USAGE,
            dcAuth::PERMISSION_ADMIN,
        ]),
        dcCore::app()->blog->id
    )
);

/* Add behavior callback */
dcCore::app()->addBehavior('coreMediaConstruct', [dcImageMetaExtender::class, 'coreMediaConstruct']);

class dcImageMetaExtender
{
    public static function transformArray()
    {
        return array(__('None') => 0, __('gpsAsString') => 1, __('gpsAsNumber') => 2, __('iptcDate') => 3);
    }

    protected static function photoMetaExif($file, $meta)
    {
#myLog::log('file: '.$file);
        $data[] = @exif_read_data($file, 'ANY_TAG');
        getimagesize($file, $info);
        if (is_array($info)) {
            $data[] = @iptcparse($info["APP13"]);
#myLog::log('found IPTC!');
        }
        foreach ($data as $exif) {
#myLog::log('exif: '.print_r($exif, true));

            // Now for every setting set
            foreach (unserialize(dcCore::app()->blog->settings->dcImageMetaExtender->extendInfo) as $value) {
#myLog::log('looking for exif property: '.$value['exif']);
                if (array_key_exists($value['exif'], $exif)) {
                    switch ($value['transform']) {
                    case 1:
                    case 2:
                        // GPS coordinates (case 1: as string; case 2: as number);
#myLog::log('adding gps coordinates');
                        $isLat = preg_match('#atitude#ui', $value['exif']);
#myLog::log('EXIF GPS: '.print_r($exif[$value['exif']], true).' / ' . print_r($exif[$value['exif'].'Ref'], true));
                        $coord = self::getGps($exif[$value['exif']], $exif[$value['exif'].'Ref'], $isLat, $value['transform'] == 1);
                        $meta->addChild($value['name'], $coord);
                        break;
                    case 3:
                        // IPTC date (input format: YYYYMMDD; output: YYYY-MM-DD)
#myLog::log('adding IPTC date');
                        $date = $exif[$value['exif']][0];
#myLog::log('input: ' . $date);
                        $result = implode('-', array(substr($date, 0, 4), substr($date, 4, 2), substr($date, 6, 2)));
#myLog::log('output: ' . $result);
                        $meta->addChild($value['name'], $result);
                        break;
                    case 0:
                    default:
                        // No transform, simply add the exif property value
#myLog::log('adding simple exif property');
                        if (is_array($exif[$value['exif']])) {
                            $meta->addChild($value['name'], implode(', ', $exif[$value['exif']]));
                        } else {
                            $meta->addChild($value['name'], $exif[$value['exif']]);
                        }
                    }
                }
            }
        }
    }

    protected static function getGps($exifCoord, $hemi, $isLat, $asString = false)
    {
        // Source: http://stackoverflow.com/questions/2526304/php-extract-gps-exif-data/2572991#2572991
#myLog::log('ExifCoord: '.print_r($exifCoord, true));
#myLog::log('Count: '.count($exifCoord));
        $degrees = !empty($exifCoord)    ? self::gps2Num($exifCoord[0]) : 0;
        $minutes = count($exifCoord) > 1 ? self::gps2Num($exifCoord[1]) : 0;
        $seconds = count($exifCoord) > 2 ? self::gps2Num($exifCoord[2]) : 0;
#myLog::log("Deg: $degrees / Min: $minutes / Sec: $seconds / Hemi: $hemi");

        // Carrying down the floating part of degrees and minnutes
        $minutes = $minutes + ($degrees - (int)$degrees) * 60;
        $degrees = (int)$degrees;
        $seconds = $seconds + ($minutes - (int)$minutes) * 60;
        $minutes = (int)$minutes;
#myLog::log("Deg: $degrees / Min: $minutes / Sec: $seconds / Hemi: $hemi");

        $flip = 1;
        if ($hemi == 'N' || $hemi == 'E' || $hemi == 'W' || $hemi == 'S') {
            $flip = ($hemi == 'W' || $hemi == 'S') ? -1 : 1;
        } else {
            if ($degrees > 0) {
                $hemi = $isLat ? 'N' : 'E';
            } else {
                $hemi = $isLat ? 'S' : 'W';
            }
        }

        if ($asString) {
            $seconds = round($seconds, 2);
            return $degrees . '°'. " $minutes' $seconds\" $hemi";
        } else {
            return $flip * ($degrees + $minutes / 60 + $seconds / 3600);
        }
    }

    protected static function gps2Num($coordPart)
    {
        $parts = explode('/', $coordPart);
        if (empty($parts)) {
            return 0;
        }
        if (count($parts) == 1) {
            return $parts[0];
        }
        return floatval($parts[0]) / floatval($parts[1]);
    }

    # Behaviour
    public static function coreMediaConstruct($media)
    {
        $media->addFileHandler('image/jpeg', 'create', [dcImageMetaExtender::class, 'imageMetaExtend']);
    }

    public static function imageMetaExtend($cur, $f, $id)
    {
        $file = dcCore::app()->media->getPwd().'/'.$f;
#myLog::log('file: '.$file.' / id: '.$id);
        if (!file_exists($file)) {
            return false;
        }

#myLog::log('cur: '.print_r($cur, true));
        $rs = new dcRecord(dcCore::app()->con->select(
            'SELECT media_id, media_path, media_title, '.
            'media_file, media_meta, media_dt, media_creadt, '.
            'media_upddt, media_private, user_id '.
            'FROM '.dcCore::app()->prefix.dcMedia::MEDIA_TABLE_NAME.' '.
            'WHERE media_id = '.$id.' '));

        $c = dcCore::app()->con->openCursor(dcCore::app()->prefix.dcMedia::MEDIA_TABLE_NAME);

        while ($rs->fetch()) {
            $meta = $rs->media_meta;
            $meta = @simplexml_load_string($meta);
#myLog::log('meta: '.print_r($meta, true));
            dcImageMetaExtender::photoMetaExif($file, $meta);
#myLog::log('meta: '.print_r($meta, true));
            $c->media_meta = $meta->asXML();
#myLog::log("id: $id / ".'xml: '.$c->media_meta);
            $c->update('WHERE media_id = '.$id);
        }
    }
}
